/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation.simple.rdf;

import javax.annotation.Nonnull;
import it.tidalwave.util.As;
import it.tidalwave.util.AsException;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.observation.Observation;
import it.tidalwave.observation.ObservationItem;
import it.tidalwave.observation.Observer;
import it.tidalwave.semantic.io.spi.AsStatementMarshaller;
import it.tidalwave.semantic.io.spi.StatementMarshaller.Context;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class SimpleObservationMarshaller extends AsStatementMarshaller
  {
    @Override @Nonnull
    public void marshal (final @Nonnull As entity, final @Nonnull Context context)
      {
        super.marshal(entity, context);
        final Observation observation = (Observation)entity;
        context.addStatement(observation, ObservationVocabulary.RDF_TYPE, ObservationVocabulary.OBS_OBSERVATION);
        context.addStatement(observation, ObservationVocabulary.DC_DATE, observation.getDate());
        context.addStatement(observation, ObservationVocabulary.OBS_MADE_AT, observation.getLocation());
        context.addStatement(observation, ObservationVocabulary.DC_PUBLISHER, observation.getSource());

        for (final Observer observer : observation.findObservers().results())
          {
            context.addStatement(observation, ObservationVocabulary.DC_CREATOR, observer);
          }

        for (final ObservationItem observationItem : observation.findObservationItems().results())
          {
            context.addStatement(observation, ObservationVocabulary.SKOS_NARROWER, observationItem);
          }

        try
          {
            context.addStatement(observation.as(Media.class), ObservationVocabulary.FOAF_TOPIC, observation);
          }
        catch (AsException e)
          {
            // ok, no attachment
          }
      }
  }
