/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation.simple.rdf;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import it.tidalwave.util.As;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.role.spi.DefaultDisplayable;
import it.tidalwave.role.spi.DefaultIdentifiable;
import it.tidalwave.semantic.io.spi.StatementUnmarshallerSupport;
import it.tidalwave.observation.simple.SimpleSource;
import it.tidalwave.semantic.io.impl.Converter;
import org.openrdf.model.Statement;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class SimpleSourceUnmarshaller extends StatementUnmarshallerSupport
  {
    @Override @Nonnull
    public As unmarshal (final @Nonnull List<Statement> statements, final @Nonnull Context context)
      {
        // TODO: what about retrieving the observationSet and calling findOrCreate(?)
        final List<Object> capabilities = new ArrayList<Object>();
        capabilities.add(new DefaultIdentifiable(Converter.valueToId(statements.get(0).getSubject())));

        try
          {
            final String displayName = findStatementWithPredicate(statements, ObservationVocabulary.RDFS_LABEL).getObject().stringValue();
            capabilities.add(new DefaultDisplayable(displayName, "SimpleSource"));
          }
        catch (NotFoundException e)
          {
            // ok. no Displayable
          }

        addCapabilities(capabilities, statements, ObservationVocabulary.FOAF_HOME_PAGE);

        return new SimpleSource(capabilities.toArray());
      }
  }
